/**************************************************************************** 
*  Copyright 2007-2009 C&H Technologies, Inc.  All Rights Reserved.					    * 
*****************************************************************************/
/****************************************************************************
*
*	ma203_ansi.c  
*
*	ANSI-C source module for the MA203 Event Detector M-module.  The ma203 is 
*	manufactured and supported by C&H Technologies, Inc.
*
* This file contains high level functions to control a MA203 Event Detector 
*	M-module from C&H Technologies, Inc.  This file is written entirely in 
*	ANSI-C so that is may be compiled in any environment with a supporting 
*	ANIS-C compiler It uses the APIS library to perform instrument 
*	communications.  
*
*	This file is organized such that modifications are easy.  All functions 
*	that might need to be modified are located at the end of this file.  
*	Comments are provided that explain tasks of the function and how it might 
*	be modified.  All other function should not need to be modified.
*
*	C&H Part Number: 11028109
*
*	Revision History
*	-	Original Release Version 4.0 - All previos versions where legacy VXIpnp 
*		drivers.  See the legacy VXIpnp source code for details on previous 
*		versions.
*	-	Minor modification to support the release of CHWIN 1.0.
*	-	Added ReadFifoBuffer
*	-	Release 4.1 (11-08-07 - David Clark)
*	-	Added "if(handle->apis_handle != NULL)" to ma203_Close to protect from
*		someone calling close when apis_open has failed.  (12-12-07 David Clark)
*	-	Fixed a bug in m203_Init() in which the accessory module options 4 & 5 
*		where not being initialized.  This caused the thresholds to be incorrect
*		and to exhibit unexpected behavior.  Added a call to 
*		ma203_AccessoryOption4Init().  (08/12/08 - David Clark)
*	-	Added a check for option 5 in functions ma203_AccessoryOption4Init() and 
*		ma203_AccessoryOption4WriteData().  Previously option 5 would not work.
*		(08/12/08 - David Clark)
*	-	Release 4.2 (8-12-08 - David Clark)
*	-	Modified driver to support the AM104-6 Accessory Module (option 6).  The
*		major change was in ma203_SetInputThreshold().  In addition, the static 
*		functions ma203_AccessoryOption6Init() and ma203_AccessoryOption6WriteData() 
*		were a added  (2/6/2009 - David Clark)
*	-	Added a call to ma203_AccessoryOption4Init() and ma203_AccessoryOption6Init()
*		in the reset function so that the threshold is reset to 0 whenever 
*		ma203_Reset() is called (2/6/2009 - David Clark)
*	-	Release 4.3 (2/25/09 - David Clark)
*
*
*****************************************************************************/
#include <ansi_c.h>
#include "ma203_ansi.h"

/**************************************************************************** 
*  Non Exported Defines
*****************************************************************************/
/*  Driver Revision  */
#define		MA203_DRIVER_REV			"4.3"

/*  Misc. Defined  */
#define		MA203_NUM_ACCESSORY_OPTIONS		6	



/**************************************************************************** 
*  Error Checking Macros
*****************************************************************************/
#ifndef checkParamRange
#define  checkParamRange(value, min, max, param)		if ((value > max) || (value < min)) { \
																											error = MA203_ERROR_PARAMETER1 + param-1;\
																											goto Error; \
																										} else 
#endif																				
																				
#ifndef checkParamPointer
#define  checkParamPointer(pointer, param)		if (pointer == NULL) { \
																							error = MA203_ERROR_PARAMETER1 + param-1;\
																							goto Error; \
																							} else 
#endif																				

																				
#ifndef checkErr
#define checkErr(fCall)      if (error = (fCall), (error = (error < 0) ? error : MA203_SUCCESS)) \
                                 {goto Error;}  else
#endif

#ifndef apisCheckErr
#define apisCheckErr(fCall)      if (error = (fCall), (error = (error != 0) ? error : MA203_SUCCESS)) \
                                 {	handle->apis_error = error;\
																		error = MA203_ERROR_APIS; \
																		goto Error;}  else
#endif

#ifndef apisCheckErrInit
#define apisCheckErrInit(fCall)      if (error = (fCall), (error = (error != 0) ? error : MA203_SUCCESS)) \
                                 {	(*handle)->apis_error = error;\
																		error = MA203_ERROR_APIS; \
																		goto Error;}  else
#endif

#ifndef round8bit
#define round8bit(x) ((x)>=0?(UInt8)((x)+0.5):(UInt8)((x)-0.5))
#endif
																				
#ifndef round16bit
#define round16bit(x) ((x)>=0?(UInt16)((x)+0.5):(UInt16)((x)-0.5))
#endif

/*************************************************************************** 
*  Defined Structures
*****************************************************************************/
typedef struct{
	Real64 debounce_time_us;
	UInt8 debounce_value;
}ma203_DebounceRangeTable;



/**************************************************************************** 
*  Constants
*****************************************************************************/
static const ma203_DebounceRangeTable standard_debounce_table[16] = {	{0, 0x0},
																																				{8, 0x1},
																																				{16, 0x2},
																																				{32, 0x3},
																																				{64, 0x4},
																																				{128, 0x5},
																																				{256, 0x6},
																																				{512, 0x7},
																																				{1000, 0x8},
																																				{2000, 0x9},
																																				{4000, 0xA},
																																				{8000, 0xB},
																																				{16000, 0xC},
																																				{32000, 0xD},
																																				{64000, 0xE},
																																				{128000, 0xF}};
																							
static const ma203_DebounceRangeTable high_speed_debounce_table[16] = {	{0, 0x0},
																																					{0.2, 0x1},
																																					{0.4, 0x2},
																																					{0.8, 0x3},
																																					{1.6, 0x4},
																																					{3.2, 0x5},
																																					{6.4, 0x6},
																																					{12.8, 0x7},
																																					{25.6, 0x8},
																																					{51.2, 0x9},
																																					{102, 0xA},
																																					{205, 0xB},
																																					{410, 0xC},
																																					{819, 0xD},
																																					{1640, 0xE},
																																					{3280, 0xF}};



static const char *accessoryOptions[MA203_NUM_ACCESSORY_OPTIONS] = {	"+5V Optocoupled Input",
																																				"+28V Optocoupled Input",
																																				"Contact Closure Sensing Input",
																																				"+25V Programmable Threshold Input",
																																				"-2V to +30V Programmable Threshold Input",
																																				"Isolated Programmable Threshold & RS422 Monitoring"
																																			};


static const char *errorMessageTable[MA203_SPECIFIC_ERROR_END - MA203_SPECIFIC_ERROR_BASE] = 
						{	"The value passed to the function has been reserved for future use.",
							"You cannot mix high speed and standard debounce times. All 4 groups must be in the same range, either high speed or standard.",
							"This function cannot be performed while the instrument is running.",
							"The installed accessory module does not support this function.",
							"An invalid parameter was passed to the function in parameter 1",
							"An invalid parameter was passed to the function in parameter 2",
							"An invalid parameter was passed to the function in parameter 3",
							"An invalid parameter was passed to the function in parameter 4",
							"The module returned an invalid ID.",
							"An error occurred in the APIS Platform Support Module.  Use the function ma203_GetApisError() to retrieve the APIS error code.",
							"The operation could not be completed because the system is out of memory."
							"The debounce time is not valid. See the funtion help for valid values.",
							"Fifo empty. No valid data was found in the fifo.",
						};

static const char *warningMessageTable[MA203_SPECIFIC_WARNING_END - MA203_SPECIFIC_WARNING_BASE] = 
						{	"There were less samples stored in the FIFO than what was requested."
						};

/**************************************************************************** 
*  Non Exported Functions
*****************************************************************************/
static Int32 ma203_GetDebounceValue(Real64 debounce_time, UInt8 *debounce_value, Bool *debounce_select);
static Int32 ma203_AccessoryOption4Init (MA203_HANDLE handle);
static Int32 ma203_AccessoryOption4WriteData (MA203_HANDLE handle, Int32 chip_select, UInt8 address, UInt8 data);
static Int32 ma203_AccessoryOption6Init (MA203_HANDLE handle);
static Int32 ma203_AccessoryOption6Reset (MA203_HANDLE handle);
static Int32 ma203_AccessoryOption6WriteData (MA203_HANDLE handle, UInt8 dac_output, UInt16 data);
static Int32 ma203_IDWriteByte(MA203_HANDLE handle, UInt8 value);
static Int32 ma203_IDWriteBit(MA203_HANDLE handle, Bool value);
static Int32 ma203_IDReadByte(MA203_HANDLE handle, UInt8 *value);
static Int32 ma203_IDReadBit(MA203_HANDLE handle, Bool *value);



/*****************************************************************************
*	ma203_RevisionQuery
*
*	Returns the version number of this driver.  
******************************************************************************/
Int32 ma203_RevisionQuery (Int8 driver_rev[])
{	Int32 error = MA203_SUCCESS;

/*  Check parameters  */
	checkParamPointer(driver_rev, 2);
	
	sprintf(driver_rev, "%s", MA203_DRIVER_REV);
Error:
	return error;
}



/*****************************************************************************
*	ma203_ErrorMessage
*
*	Returns a text description of the specified status code.  
******************************************************************************/
Int32 ma203_ErrorMessage (Int32 statusCode, Int8 errorMessage[])
{	Int32 error = MA203_SUCCESS;
	
/*  Check parameters  */
	checkParamPointer(errorMessage, 3);

/*  Check driver defined error codes  */
	if((statusCode >= MA203_SPECIFIC_ERROR_BASE) && (statusCode < MA203_SPECIFIC_ERROR_END))
	{	strcpy(errorMessage, errorMessageTable[statusCode - MA203_SPECIFIC_ERROR_BASE]);
	}
	else if((statusCode >= MA203_SPECIFIC_WARNING_BASE) && (statusCode < MA203_SPECIFIC_WARNING_END))
	{	strcpy(errorMessage, warningMessageTable[statusCode - MA203_SPECIFIC_WARNING_BASE]);
	}
	else if (statusCode == 0)
		sprintf(errorMessage, "No Error Occured");
	else
		sprintf(errorMessage, "Unknown Error Code [0x%x]", statusCode);
		
Error:
	return error;
}


/*****************************************************************************
*	ma203_GetIdentWord
*
*	Reads the specified offset from the ma203's ID space and returns
*	the result.
******************************************************************************/
Int32 ma203_GetIdentWord (MA203_HANDLE handle, UInt16 offset, UInt16 *value)
{	Int32 error = MA203_SUCCESS;
	UInt8 data;

/*  Check parameters  */
	checkParamPointer(value, 3);

	*value = 0;
	
/*  Send the read command to the serial EEPROM  */	
	checkErr(ma203_IDWriteByte(handle, 0x80 | offset));
	
/*  Read upper byte  */	
	checkErr(ma203_IDReadByte(handle, &data));
	*value = data << 8;
	
/*  Read lower byte  */	
	checkErr(ma203_IDReadByte(handle, &data));
	*value |= data;
	
Error:	
	ma203_WriteRegister(handle, MA203_ID_ACCESS_REGISTER, 0);
	return error;
}



/*****************************************************************************
*	ma203_SetPolarity
*
*	Writes the specified value to the polarity register
******************************************************************************/
Int32 ma203_SetPolarity (MA203_HANDLE handle, UInt16 polarityMask)
{	Int32 error = MA203_SUCCESS;
	checkErr(ma203_WriteRegister(handle, MA203_POLARITY_REGISTER, polarityMask));
Error:
	return error;
}


/*****************************************************************************
*	ma203_GetPolarity
*	
*	Returns the current value of the polarity register
******************************************************************************/
Int32 ma203_GetPolarity (MA203_HANDLE handle, UInt16 *polarityMask)
{	Int32 error = MA203_SUCCESS;

/*  Check parameters  */
	checkParamPointer(polarityMask, 2);
	
	checkErr(ma203_ReadRegister(handle, MA203_POLARITY_REGISTER, polarityMask));
Error:
	return error;
}


/*****************************************************************************
*	ma203_SetWatchMask 
*
*	Writes the specified value to the watch register
******************************************************************************/
Int32 ma203_SetWatchMask (MA203_HANDLE handle, UInt16 watchMask)
{	Int32 error = MA203_SUCCESS;
	checkErr(ma203_WriteRegister(handle, MA203_WATCH_REGISTER, watchMask));
Error:
	return error;
}


/*****************************************************************************
*	ma203_GetWatchMask
*	
*	Returns the current value of the watch register
******************************************************************************/
Int32 ma203_GetWatchMask (MA203_HANDLE handle, UInt16 *watchMask)
{	Int32 error = MA203_SUCCESS;
	
/*  Check parameters  */
	checkParamPointer(watchMask, 2);
	
	checkErr(ma203_ReadRegister(handle, MA203_WATCH_REGISTER, watchMask));
Error:
	return error;
}


/*****************************************************************************
*	ma203_ConfigureDebounce
*
*	Configures the ma203's debounce to the specified values.  The debounce 
*	times must all be either a high speed value or a standard value.  If 
*	one value is a high speed value and another is a standard value this 
*	function will return an error.
******************************************************************************/
Int32 ma203_ConfigureDebounce (MA203_HANDLE handle, Real64 channels0_3, Real64 channels4_7, Real64 channels8_11,
                                             Real64 channels12_15)
{	Int32 error = MA203_SUCCESS;
	UInt8 value0_3, value4_7, value8_11, value12_15;
	Bool debounce_select0_3, debounce_select4_7, debounce_select8_11, debounce_select12_15;
	UInt16 value = 0;

/*  Get the register value and the high/low speed selection for the debounce time  */	
	checkErr(ma203_GetDebounceValue(channels0_3, &value0_3, &debounce_select0_3));
	checkErr(ma203_GetDebounceValue(channels4_7, &value4_7, &debounce_select4_7));
	checkErr(ma203_GetDebounceValue(channels8_11, &value8_11, &debounce_select8_11));
	checkErr(ma203_GetDebounceValue(channels12_15, &value12_15, &debounce_select12_15));
	
/*  If all debounce times are in the same category (low speed or high speed) then set the values  */
	if((debounce_select0_3 & debounce_select4_7 & debounce_select8_11 & debounce_select12_15) > 0)
	{	value = value0_3 | (value4_7 << 4) | (value8_11 << 8) | (value12_15 << 12);		
		checkErr(ma203_WriteRegister(handle, MA203_DEBOUNCE_CONTROL_REGISTER, value));
		
	/*  A debounce time of 0 can be either high or low speed  */
		if((debounce_select0_3 & 0x2) > 0)
		{	
			checkErr(ma203_ReadRegister(handle, MA203_CONTROL_STATUS_REGISTER, &value));
			checkErr(ma203_WriteRegister(handle, MA203_CONTROL_STATUS_REGISTER, (value | MA203_DEBOUNCE_CONTROL_BIT & ~MA203_STATUS_BITS)));
		}
		else
		{
			checkErr(ma203_ReadRegister(handle, MA203_CONTROL_STATUS_REGISTER, &value));
			checkErr(ma203_WriteRegister(handle, MA203_CONTROL_STATUS_REGISTER, (value & ~MA203_DEBOUNCE_CONTROL_BIT & ~MA203_STATUS_BITS)));
		}
	}
	else
		error = MA203_ERROR_DEBOUNCE_SELECT_MISMATCH;
	

Error:
	return error;
}


/*****************************************************************************
*	ma203_ConfigureSampleClock
*
*	Configures the clock the the ma203 uses for sampling.  Sets up the 
*	clock control register according to values specified in the parameters.
******************************************************************************/
Int32 ma203_ConfigureSampleClock (MA203_HANDLE handle, Int32 clockSource, Int32 internalClockRate, 
                                                Int32 clockPrescaler,
                                                Bool clockOutSelect,
                                                Bool clockOutEnable,
                                                Bool triggerOutSelect)
{	
	Int32 error = MA203_SUCCESS;
	UInt16 value = 0;

	checkParamRange(clockSource, 0, 7, 2);
	checkParamRange(clockOutEnable, 0, 1, 6);
	
	if(clockSource == 1)
	{ error = MA203_ERROR_RESERVED_VALUE;
		goto Error;
	}
	checkParamRange(internalClockRate, 0, 3, 3);
	checkParamRange(clockPrescaler, 0, 7, 4);
	checkParamRange(clockOutSelect, 0, 1, 5);
	checkParamRange(triggerOutSelect, 0, 1, 7);
	
/*  Form register value and write it  */
	value = clockSource | (internalClockRate << 8) | (clockPrescaler << 4) | (clockOutSelect << 14) | (clockOutEnable << 13) | (triggerOutSelect << 12); 		
	checkErr(ma203_WriteRegister(handle, MA203_CLOCK_CONTROL_REGISTER, value));
Error:
	return error;
}

/*****************************************************************************
*	ma203_ConfigureRunSource
*	
*	Sets the source of the signal that the ma203 uses to run and stop.
*	Writes to bits 4-6 of the control status register
******************************************************************************/
Int32 ma203_ConfigureRunSource (MA203_HANDLE handle, Int32 runSource)
{	Int32 error = MA203_SUCCESS;
	UInt16 value = 0;

	checkParamRange(runSource, 0, 7, 2);
	

	if(runSource == 1)
	{ error = MA203_ERROR_RESERVED_VALUE;
		goto Error;
	}
	
	checkErr(ma203_ReadRegister(handle, MA203_CONTROL_STATUS_REGISTER, &value));
	value &= ~MA203_RUN_SEL_BITS & ~MA203_STATUS_BITS;  // Do not write to status bits
	value |= runSource << 4;
	checkErr(ma203_WriteRegister(handle, MA203_CONTROL_STATUS_REGISTER, value));
	
Error:
	return error;
}


/*****************************************************************************
*	ma203_StoreAll
*	
*	Sets or clears the store all bit in the control satuts register
******************************************************************************/
Int32 ma203_StoreAll (MA203_HANDLE handle, Bool storeAll)
{	Int32 error = MA203_SUCCESS;
	UInt16 value = 0;

/*  Check parameters  */
	checkParamRange(storeAll, 0, 1, 2);

	checkErr(ma203_ReadRegister(handle, MA203_CONTROL_STATUS_REGISTER, &value));
	value &= ~MA203_STORE_ALL_BIT & ~MA203_STATUS_BITS; // Do not write to status bits
	value |= storeAll << 7;
	checkErr(ma203_WriteRegister(handle, MA203_CONTROL_STATUS_REGISTER, value));

Error:
	return error;
}



/*****************************************************************************
*	ma203_EnableInterrupts 
*
*	Configures the ma203 to produce interrupts.  This function does not
*	configure the conditions on which interrupts occur.
******************************************************************************/
Int32 ma203_EnableInterrupts (MA203_HANDLE handle, Bool interruptEnable, UInt8 interruptVector)
{	
	Int32 error = MA203_SUCCESS;
	UInt16 value = 0;

/*  Check parameters  */
	checkParamRange(interruptEnable, 0, 1, 2);
	checkParamRange(interruptVector, 0, 0xFF, 4);

/*  Configure the Ma203 ROAK interrupts with the given vector  */	
	checkErr(ma203_ReadRegister(handle, MA203_MASTER_INTERRUPT_REGISTER, &value));
	value &= 0xF100;
	value |= ((interruptEnable << 9) | interruptVector | MA203_INTERRUPT_TYPE_BIT);
	checkErr(ma203_WriteRegister(handle, MA203_MASTER_INTERRUPT_REGISTER, value));
	
Error:
	return error;
}



/*****************************************************************************
*	ma203_ConfigIntConditions
*
*	Configure the conditions on which the ma203 will produce an interrupt.
******************************************************************************/
Int32 ma203_ConfigIntConditions (MA203_HANDLE handle, Bool dataStored, Bool fifoFull, Bool fifoHalfFull,
                                               Bool timeStampRollover,
                                               Bool pattern,
                                               UInt16 interruptDefinition,
                                               UInt16 channelInterruptEnable)
{	Int32 error = MA203_SUCCESS;
	UInt16 value = 0;
	Bool interrupt_enabled = MA203_FALSE;
	UInt16 interrupt_pending = 0;
	
/*  Check parameters  */
	checkParamRange(dataStored, 0, 1, 2);
	checkParamRange(fifoFull, 0, 1, 3);
	checkParamRange(fifoHalfFull, 0, 1, 4);
  checkParamRange(timeStampRollover, 0, 1, 5);
  checkParamRange(pattern, 0, 1, 6);

/*  Save any interrupts pending prior to entering this function  */
	checkErr(ma203_ReadRegister(handle, MA203_INTERRUPT_PENDING_REGISTER, &interrupt_pending));

/*  Disable interrupts while changing interrupt conditions to avoid errant interrupts	 */
	checkErr(ma203_ReadRegister(handle, MA203_MASTER_INTERRUPT_REGISTER, &value));
	if(value & MA203_MASTER_INTERRUPT_ENABLE_BIT) 
	{	value = value & ~MA203_MASTER_INTERRUPT_ENABLE_BIT;
		checkErr(ma203_WriteRegister(handle, MA203_MASTER_INTERRUPT_REGISTER, value));
		interrupt_enabled = MA203_TRUE;
	}

/*  Configure the Ma203  */	
	checkErr(ma203_WriteRegister(handle, MA203_INTERRUPT_DEFINITION_REGISTER, interruptDefinition));
	checkErr(ma203_WriteRegister(handle, MA203_CHANNEL_INTERRUPT_REGISTER, channelInterruptEnable));
	
	value &= 0x06FF;
	value |= (dataStored << 15) | (fifoFull << 14) | (fifoHalfFull << 13) | (timeStampRollover << 12) | (pattern << 8);
	checkErr(ma203_WriteRegister(handle, MA203_MASTER_INTERRUPT_REGISTER, value));

/*  Clear interrupts that weren't pending prior to entering this function  */
	checkErr(ma203_WriteRegister(handle, MA203_INTERRUPT_PENDING_REGISTER, ~interrupt_pending));  //clear all interrupts that occured
																																																		//as a result of this configuration
/*  Re-enable interrupts if they were enabled prior to entering this function  */
	if(interrupt_enabled == MA203_TRUE)
	{	checkErr(ma203_ReadRegister(handle, MA203_MASTER_INTERRUPT_REGISTER, &value));
		checkErr(ma203_WriteRegister(handle, MA203_MASTER_INTERRUPT_REGISTER, value | MA203_MASTER_INTERRUPT_ENABLE_BIT));
	}
	
Error:
	return error;
}



/*****************************************************************************
*	ma203_ClearInterrupts
*
*	Clears interrupt pendings for the channels specified by the
*	clearInterruptMask parameter.
******************************************************************************/
Int32 ma203_ClearInterrupts (MA203_HANDLE handle, UInt16 clearInterruptMask)
{	Int32 error = MA203_SUCCESS;
	checkErr(ma203_WriteRegister(handle, MA203_INTERRUPT_PENDING_REGISTER, clearInterruptMask));
Error:
	return error;
}


/*****************************************************************************
*	ma203_GetInterruptPending
*
*	Returns the value of the interrupt pending register
******************************************************************************/
Int32 ma203_GetInterruptPending (MA203_HANDLE handle, UInt16 *interruptPendingMask)
{	Int32 error = MA203_SUCCESS;

/*  Check parameters  */
	checkParamPointer(interruptPendingMask, 2);
	checkErr(ma203_ReadRegister(handle, MA203_INTERRUPT_PENDING_REGISTER, interruptPendingMask));
Error:
	return error;
}


/*****************************************************************************
*	ma203_ReEnableInterrupts
*
*	Set the master interrupt enable bit to 1.  This bit is cleared on an 
*	interrupt and must be reset for future interrupts to occur.
******************************************************************************/
Int32 ma203_ReEnableInterrupts (MA203_HANDLE handle)
{	Int32 error = MA203_SUCCESS;
	UInt16 value = 0;
//	ma203_ATTRIBUTES *attributes;


/*  Set the master interrupt enable bit to 1  */
	checkErr(ma203_ReadRegister(handle, MA203_MASTER_INTERRUPT_REGISTER, &value));
	value |= MA203_INTERRUPT_TYPE_BIT | MA203_MASTER_INTERRUPT_ENABLE_BIT;
	checkErr(ma203_WriteRegister(handle, MA203_MASTER_INTERRUPT_REGISTER, value));
	
Error:
	return error;
}

																																		
/*****************************************************************************
*	ma203_AccessoryModuleOption
*
*	Returns the option number of the installed accessory module and
*	a description of the option.
******************************************************************************/
Int32 ma203_AccessoryModuleOption (MA203_HANDLE handle, Int32 *optionNumber, Int8 *optionDescription)
{	Int32 error = MA203_SUCCESS;
	UInt16 value = 0;
	
/*  Check parameters  */
	checkParamPointer(optionNumber, 2);
	
/*  Read the option number from the instrument  */
	checkErr(ma203_ReadRegister(handle, MA203_ACCESSORY_MODULE_REGISTER, &value));
	*optionNumber = (value >> 12) & 0x000F;
	
/*  Set the option description string if it is not a NULL pointer  */	
	if((*optionNumber <= MA203_NUM_ACCESSORY_OPTIONS) && (*optionNumber > 0) && (optionDescription != NULL))
	{ strcpy (optionDescription, accessoryOptions[(*optionNumber)-1]);
	}
Error:
	return error;
}


/*****************************************************************************
*	ma203_SetInputThreshold
*
*	Set the input threshold for the channels specified by the channel mask 
*	to the specified value.  If option 4, 5 or 6 accessory module is not installed 
*	this function will return an error.
******************************************************************************/
Int32 ma203_SetInputThreshold (MA203_HANDLE handle, Real64 inputThreshold, UInt16 channelMask)
{	Int32 error = MA203_SUCCESS;
	Int32 accessory_module;
	UInt8 data;
	UInt16 data16;
	Int32 option;
	
	checkErr(ma203_AccessoryModuleOption (handle, &option, NULL));

	if(option == 4)
	{	checkParamRange(inputThreshold, 0, 25.5, 2);
		data = round8bit(inputThreshold/MA203_ACC_OPTION_4_RESOLUTION);
	}
	else if (option == 5)
	{	checkParamRange(inputThreshold, -2, 30.0, 2);
		data = round8bit((inputThreshold - MA203_ACC_OPTION_5_OFFSET)/MA203_ACC_OPTION_5_RESOLUTION);
	}
	else if (option == 6)
	{	checkParamRange(inputThreshold, 0, 40.0, 2);
		data16 = round16bit(inputThreshold/MA203_ACC_OPTION_6_RESOLUTION);
		if(data16 > 0x0FFF)
			data16 = 0x0FFF;
	}
	else
	{	error = MA203_ERROR_INVALID_ACCESSORY_FUNCTION;
		goto Error;
	}

	if((option ==4) || (option == 5))
	{
		if((channelMask & 0x00FF) != 0)
			checkErr(ma203_AccessoryOption4WriteData(handle, 0, (UInt8)(channelMask & 0x00FF), data)); 
	
		if((channelMask & 0xFF00) != 0)
			checkErr(ma203_AccessoryOption4WriteData(handle, 1, (UInt8)((channelMask >> 8) & 0x00FF), data)); 
	}
	else	//option == 6
	{	if(channelMask & 0x1)
			checkErr(ma203_AccessoryOption6WriteData (handle, 0, data16));

		if(channelMask & 0x2)
			checkErr(ma203_AccessoryOption6WriteData (handle, 1, data16));
	}	

Error:
	return error;
}

/*****************************************************************************
*	ma203_Run
*
*	Sets the run bit in the control status register.  This will begin sampling.
******************************************************************************/
Int32 ma203_Run (MA203_HANDLE handle)
{	Int32 error = MA203_SUCCESS;
	UInt16 value;
	checkErr(ma203_ReadRegister(handle, MA203_CONTROL_STATUS_REGISTER, &value));
	checkErr(ma203_WriteRegister(handle, MA203_CONTROL_STATUS_REGISTER, (value | MA203_RUN_BIT & ~MA203_STATUS_BITS))); // Do not write to status bits
	
Error:
	return error;
}

/*****************************************************************************
*	ma203_Stop
*
*	Clears the run bit inthe contorl status register.  This will stop sampling.
******************************************************************************/
Int32 ma203_Stop (MA203_HANDLE handle)
{	Int32 error = MA203_SUCCESS;
	UInt16 value;
	checkErr(ma203_ReadRegister(handle, MA203_CONTROL_STATUS_REGISTER, &value));
	checkErr(ma203_WriteRegister(handle, MA203_CONTROL_STATUS_REGISTER, (value & ~MA203_RUN_BIT & ~MA203_STATUS_BITS)));// Do not write to status bits
Error:
	return error;
}

/*****************************************************************************
*	ma203_SingleSample
*
*	Set the sample bit in the control status register.  This force one
*	sample and store to occur.  If this function is called while the
*	run bit is set, it will return an error.
******************************************************************************/
Int32 ma203_SingleSample (MA203_HANDLE handle)
{	Int32 error = MA203_SUCCESS;
	UInt16 value;
	checkErr(ma203_ReadRegister(handle, MA203_CONTROL_STATUS_REGISTER, &value));

/*  Make sure the MA203 is not running  */	
	if(value & MA203_RUN_BIT)
	{	error = MA203_ERROR_RUNNING;
		goto Error;
	}
	else
		checkErr(ma203_WriteRegister(handle, MA203_CONTROL_STATUS_REGISTER, (value | MA203_SINGLE_SAMPLE_BIT & ~MA203_STATUS_BITS)));// Do not write to status bits
																																																										
Error:
	return error;
}


/*****************************************************************************
*	ma203_CurrentValue
*
*	Returns the value of the current value register
******************************************************************************/
Int32 ma203_CurrentValue (MA203_HANDLE handle, UInt16 *currentValue)
{	Int32 error = MA203_SUCCESS;

/*  Check parameters  */
	checkParamPointer(currentValue, 2);
	
	checkErr(ma203_ReadRegister(handle, MA203_CURRENT_VALUE_REGISTER, currentValue));
Error:
	return error;
}


/*****************************************************************************
*	ma203_LastStoredValue
*
*	Returns the value of the last value stored register
******************************************************************************/
Int32 ma203_LastStoredValue (MA203_HANDLE handle, UInt16 *lastStoredValue)
{	Int32 error = MA203_SUCCESS;

/*  Check parameters  */
	checkParamPointer(lastStoredValue, 2);
	
	checkErr(ma203_ReadRegister(handle, MA203_LAST_VALUE_STORED_REGISTER, lastStoredValue));
Error:
	return error;
}


/*****************************************************************************
*	ma203_ReadFifo
*
*	Reads the next time stamp/data pair from the fifo.  If no valid data
*	exists in the fifo this function will return an error.
******************************************************************************/
Int32 ma203_ReadFifo (MA203_HANDLE handle, UInt32 *timeStamp, UInt16 *value)
{	Int32 error = MA203_SUCCESS;
	UInt16 data;

/*  Check parameters  */
	checkParamPointer(timeStamp, 2);
	checkParamPointer(value, 3);

	checkErr(ma203_ReadRegister(handle, MA203_FIFO_DATA_REGISTER, &data));
	if (data & MA203_DATA_VALID_BIT)  //Check valid bit
	{	*timeStamp = (data << 16) & 0x7FFF0000l;
		checkErr(ma203_ReadRegister(handle, MA203_FIFO_DATA_REGISTER, &data));
		*timeStamp |= data;
		checkErr(ma203_ReadRegister(handle, MA203_FIFO_DATA_REGISTER, value));
	}
	else
	{	error = MA203_ERROR_FIFO_EMPTY;
		goto Error;
	}
	
Error:
	return error;
}



/*****************************************************************************
*	ma203_ReadFifoBuffer
*
*	Reads a buffer of time stamp/data pairs from the fifo.  If no valid data
*	exists in the fifo this function will return an error.
******************************************************************************/
Int32 ma203_ReadFifoBuffer (MA203_HANDLE handle, UInt32 count, UInt32 *time_stamp_buffer, UInt16 *data_buffer, UInt32 *num_read)
{	Int32 error = MA203_SUCCESS;
	Int32 i;
	UInt16 *buffer = NULL;;

/*  Check parameters  */
	checkParamPointer(time_stamp_buffer, 3);
	checkParamPointer(data_buffer, 4);
	checkParamPointer(num_read, 5);

	buffer = malloc(count * sizeof(UInt16) * 3);
	if(buffer == NULL)
	{	error = MA203_ERROR_OUT_OF_MEMORY;
		goto Error;
	}
	
	*num_read = 0;
	
	apisCheckErr(apis_readfifo(handle->apis_handle, 2, MA203_FIFO_DATA_REGISTER, count * 3, buffer));
	
	for(i=0; i<count; i++)
	{
		if (buffer[i*3] & MA203_DATA_VALID_BIT)  //Check valid bit
		{	time_stamp_buffer[i] = (buffer[i*3] << 16) & 0x7FFF0000l;
			time_stamp_buffer[i] |= buffer[(i*3) + 1];
			data_buffer[i] = buffer[(i*3) + 2];
			(*num_read)++;
		}
	}

	if(*num_read != count)
		error = MA203_SPECIFIC_WARNING_NUM_SAMPLES_NOT_AVAILABLE;
Error:
	if(buffer != NULL)
		free(buffer);
	return error;
}


/*****************************************************************************
*	ma203_ResetFifo
*
*	Resets the fifo.  If this function is called while the run bit is set
*	it will return an error.  This will also clear any of the fifo interrupts
******************************************************************************/
Int32 ma203_ResetFifo (MA203_HANDLE handle)
{	Int32 error = MA203_SUCCESS;
	UInt16 value;

/*  Write to fifo status bits to clear any pending interrupts but do not write to time stamp status bits  */
	checkErr(ma203_ReadRegister(handle, MA203_CONTROL_STATUS_REGISTER, &value));
	checkErr(ma203_WriteRegister(handle, MA203_CONTROL_STATUS_REGISTER, (value | MA203_RESET_FIFO_BIT & ~MA203_TIME_STAMP_ROLLOVER_BIT)));
Error:
	return error;
}

/*****************************************************************************
*	ma203_FifoUnreadCount 
*
*	Returns the value of the fifo unread count register
******************************************************************************/
Int32 ma203_FifoUnreadCount (MA203_HANDLE handle, UInt16 *unreadCount)
{	Int32 error = MA203_SUCCESS;

/*  Check parameters  */
	checkParamPointer(unreadCount, 2);

	checkErr(ma203_ReadRegister(handle, MA203_FIFO_UNREAD_COUNT_REGISTER, unreadCount));
Error:
	return error;
}


/*****************************************************************************
*	ma203_GetFifoStatus 
*
*	Returns the values of the three fifo status bits.  These bits are in
*	the control status register bits 13-15.
******************************************************************************/
Int32 ma203_GetFifoStatus (MA203_HANDLE handle, Bool *fifoFull, Bool *fifoHalfFull, Bool *dataStored)
{	Int32 error = MA203_SUCCESS;
	UInt16 value;
	
/*  Check parameters  */
	checkParamPointer(fifoFull, 2);
	checkParamPointer(fifoHalfFull, 3);
	checkParamPointer(dataStored, 4);
	
	*dataStored = 0;
	*fifoFull = 0;
	*fifoHalfFull = 0;
	
	checkErr(ma203_ReadRegister(handle, MA203_CONTROL_STATUS_REGISTER, &value));
	if(value & MA203_DATA_STORED_BIT)
		*dataStored = 1;
	if(value & MA203_FIFO_FULL_BIT)
		*fifoFull = 1;
	if(value & MA203_FIFO_HALF_FULL_BIT)
		*fifoHalfFull = 1;
Error:
	return error;
}


/*****************************************************************************
*	ma203_ResetTimeStamp 
*
*	Resets the time stamp.
******************************************************************************/
Int32 ma203_ResetTimeStamp (MA203_HANDLE handle)
{	Int32 error = MA203_SUCCESS;
	UInt16 value;

/*  Write to time stamp status bits to clear any pending interrupts but do not write to fifo status bits  */
	checkErr(ma203_ReadRegister(handle, MA203_CONTROL_STATUS_REGISTER, &value));
	checkErr(ma203_WriteRegister(handle, MA203_CONTROL_STATUS_REGISTER, (value | MA203_RESET_TIME_STAMP_BIT & ~MA203_STATUS_BITS | MA203_TIME_STAMP_ROLLOVER_BIT)));
Error:
	return error;
}


/*****************************************************************************
*	ma203_GetTimeStampStatus 
*
*	Returns the value of the time stamp rollover bit.
******************************************************************************/
Int32 ma203_GetTimeStampStatus (MA203_HANDLE handle, Bool *timeStampRollover)
{	Int32 error = MA203_SUCCESS;
	UInt16 value;
	
/*  Check parameters  */
	checkParamPointer(timeStampRollover, 2);
	
	*timeStampRollover = 0;
	
	checkErr(ma203_ReadRegister(handle, MA203_CONTROL_STATUS_REGISTER, &value));
	if(value & MA203_TIME_STAMP_ROLLOVER_BIT)
		*timeStampRollover = 1;
	
Error:
	return error;
}


/*****************************************************************************
*	ma203_ClearTimeStampStatus 
*
*	Clears the time stamp rollover bit.  This will also clear the time stamp 
*	rollover interrupts.
******************************************************************************/
Int32 ma203_ClearTimeStampStatus (MA203_HANDLE handle)
{	Int32 error = MA203_SUCCESS;
	UInt16 value;
	
/*  Write to time stamp status bits to clear any pending interrupts but do not write to fifo status bits  */
	checkErr(ma203_ReadRegister(handle, MA203_CONTROL_STATUS_REGISTER, &value));
	checkErr(ma203_WriteRegister(handle, MA203_CONTROL_STATUS_REGISTER, (value | MA203_TIME_STAMP_ROLLOVER_BIT & ~MA203_STATUS_BITS))); 	
Error:
	return error;
}


/*****************************************************************************
*	Int32 ma203_AccessoryOption4Init 
*
*	Initializes the A2D's on the option 4 accessory module.  If option 4 is
*	not installed this function will return an error.
******************************************************************************/
static Int32 ma203_AccessoryOption4Init (MA203_HANDLE handle)
{	Int32 error = MA203_SUCCESS;
	Int32 option_number;

/*  Verify that an option 4 module is installed  */
	checkErr(ma203_AccessoryModuleOption (handle, &option_number, NULL));
	if((option_number == 4) || (option_number == 5))
	{
	/* set all DACs to full-buffer mode */ 	
		checkErr(ma203_AccessoryOption4WriteData(handle, 0, 0, 0xFF));		
		checkErr(ma203_AccessoryOption4WriteData(handle, 1, 0, 0xFF));		

	/* set all DACs to zero output*/ 
		checkErr(ma203_AccessoryOption4WriteData(handle, 0, 0xFF, 0));		
		checkErr(ma203_AccessoryOption4WriteData(handle, 1, 0xFF, 0));	
	}
	else
		error = MA203_ERROR_INVALID_ACCESSORY_FUNCTION;

Error:	
	return (error);  
}


/*****************************************************************************
*	ma203_AccessoryOption4WriteData
*
*	Write data to the a2d on the option 4 accessory module.  If option 4 is
*	not installed this function will return an error.
******************************************************************************/
static Int32 ma203_AccessoryOption4WriteData (MA203_HANDLE handle, Int32 chip_select, UInt8 address, UInt8 data)
{ Int32 error = MA203_SUCCESS;
	UInt16 send_data;
	Int32 i;
	Int32 option_number;
	UInt16 value;

/*  Verify that an option 4 module is installed  */
	checkErr(ma203_AccessoryModuleOption (handle, &option_number, NULL));
	if((option_number != 4) && (option_number != 5)) 
	{	error = MA203_ERROR_INVALID_ACCESSORY_FUNCTION;
		goto Error;
	}
	else
	{
		send_data = ((address << 8) | data);

		checkErr(ma203_ReadRegister(handle, MA203_ACCESSORY_MODULE_REGISTER, &value));
		data = value & 0x00FF;	
		
	/*  raise both chip selects  */		
		data |= 0xC;
		checkErr(ma203_WriteRegister(handle, MA203_ACCESSORY_MODULE_REGISTER, data));  
		
	/*  lower CLK  */	
		data &= ~0x1;
		checkErr(ma203_WriteRegister(handle, MA203_ACCESSORY_MODULE_REGISTER, data));  

	/* lower 1 chip select  */
		data &= ~(0x1 << (2 + chip_select));
		checkErr(ma203_WriteRegister(handle, MA203_ACCESSORY_MODULE_REGISTER, data));  

		for(i=0; i<16; i++) 
		{
			if(send_data & 0x8000) 
				data |= 0x2;
			else 
				data &= ~0x2;
				
		/*  set DIN  */			
			data &= ~(0x1 << (2 + chip_select));
			checkErr(ma203_WriteRegister(handle, MA203_ACCESSORY_MODULE_REGISTER, data));  

		/*  raise CLK  */
			data |= 0x1;
			checkErr(ma203_WriteRegister(handle, MA203_ACCESSORY_MODULE_REGISTER, data));  

		/*  lower CLK  */
			data &= ~0x1;
			checkErr(ma203_WriteRegister(handle, MA203_ACCESSORY_MODULE_REGISTER, data));  
			send_data <<= 1;
		}
		
	/*  raise chip select  */
		data |= (0x1 << (2 + chip_select));
		checkErr(ma203_WriteRegister(handle, MA203_ACCESSORY_MODULE_REGISTER, data));  
	}
Error:
	return (error);
}


/*****************************************************************************
*	ma203_AccessoryOption6Init 
*
*	Initializes the A2D's on the option 6 accessory module.  If option 4 is
*	not installed this function will return an error.
******************************************************************************/
static Int32 ma203_AccessoryOption6Init (MA203_HANDLE handle)
{	Int32 error = MA203_SUCCESS;
	Int32 option_number;
	Int32 i;

/*  Verify that an option 6 module is installed  */
	checkErr(ma203_AccessoryModuleOption (handle, &option_number, NULL));
	if(option_number == 6)
	{ 
		ma203_AccessoryOption6Reset (handle);
		for(i=0; i<4; i++)
			checkErr(ma203_AccessoryOption6WriteData (handle, i, 0));
	}
	else
		error = MA203_ERROR_INVALID_ACCESSORY_FUNCTION;

Error:	
	return (error);  
}


/*****************************************************************************
*	ma203_AccessoryOption6Reset 
*
*	Initializes the A2D's on the option 6 accessory module.  If option 4 is
*	not installed this function will return an error.
******************************************************************************/
static Int32 ma203_AccessoryOption6Reset (MA203_HANDLE handle)
{	Int32 error = MA203_SUCCESS;
	Int32 option_number;
	UInt16 value;

/*  Verify that an option 6 module is installed  */
	checkErr(ma203_AccessoryModuleOption (handle, &option_number, NULL));
	if(option_number == 6)
	{ 
		checkErr(ma203_ReadRegister(handle, MA203_ACCESSORY_MODULE_REGISTER, &value));  
		checkErr(ma203_WriteRegister(handle, MA203_ACCESSORY_MODULE_REGISTER, value | 0x10));  
		apis_delay(1);
		checkErr(ma203_WriteRegister(handle, MA203_ACCESSORY_MODULE_REGISTER, value & ~0x10));  
	}
	else
		error = MA203_ERROR_INVALID_ACCESSORY_FUNCTION;

Error:	
	return (error);  
}



/*****************************************************************************
*	ma203_AccessoryOption6WriteData
*
*	Write data to the DAC on the option 6 accessory module.  If option 6 is
*	not installed this function will return an error.
*
*	The dac_output parameter select the DAC output to be programmed.
*	the DAC has 4 outputs however only DAC outputs 0 and 1 are used.
******************************************************************************/
static Int32 ma203_AccessoryOption6WriteData (MA203_HANDLE handle, UInt8 dac_output, UInt16 data)
{ Int32 error = MA203_SUCCESS;
	Int32 option_number;
	UInt16 value;		
	Int32 i;

	checkParamRange(dac_output, 0, 1, 2);
	checkParamRange(data, 0, 0x0FFF, 3);   //12-bit data
/*  Verify that an option 6 module is installed  */
	checkErr(ma203_AccessoryModuleOption (handle, &option_number, NULL));
	if(option_number != 6)
	{	error = MA203_ERROR_INVALID_ACCESSORY_FUNCTION;
		goto Error;
	}
	else
	{	
	/*  raise RESET, CS and LOADDAC and CLK */		
		value = 0x1D;
		checkErr(ma203_WriteRegister(handle, MA203_ACCESSORY_MODULE_REGISTER, value));  
	
		data |= (dac_output << 14) & 0xCFFF;
		
		value &= ~0x4;   //lower CS
		
		for(i=15; i>=0; i--)
		{	if((data >> i) & 0x1)
				value |= 0x2;
			else
				value &= ~0x2;
			checkErr(ma203_WriteRegister(handle, MA203_ACCESSORY_MODULE_REGISTER, value & ~0x1));  //lower CLK
			checkErr(ma203_WriteRegister(handle, MA203_ACCESSORY_MODULE_REGISTER, value));  			//raise CLK
		}
											
		value |= 0x4;   //raise CS
		checkErr(ma203_WriteRegister(handle, MA203_ACCESSORY_MODULE_REGISTER, value));
		
		value &= ~0x8;		//lower DAC Load
		checkErr(ma203_WriteRegister(handle, MA203_ACCESSORY_MODULE_REGISTER, value));
		
		value |= 0x8;			//Raise DAC load
		checkErr(ma203_WriteRegister(handle, MA203_ACCESSORY_MODULE_REGISTER, value));
		
		
	}
	
Error:
	return (error);
}



/*****************************************************************************
*	ma203_GetDebounceValue
*
*	Returns the value to be written to the register in order to configure 
*	ma203 with the specified debounce time.  In other words it translates 
*	from a time in us to a register value.  It also returns an indication 
*	of whether the specified debounce time is a high speed time or is one 
*	of the standard times(i.e. determines the setting of the DC bit (Address 
*	0 bit 11)
******************************************************************************/
static Int32 ma203_GetDebounceValue(Real64 debounce_time_us, UInt8 *debounce_value, Bool *debounce_select)
{	Int32 error = MA203_SUCCESS;
	Int32 i;
	
/*  Check parameters  */	
	checkParamPointer(debounce_value, 2);
	checkParamPointer(debounce_select, 3);

/*  Search for debounce time in the ranges tables  */
	for(i=0; i<16; i++)
	{	if(debounce_time_us == 0)
		{	*debounce_value = 0;
			*debounce_select = 3;
			break;
		}
		else if(debounce_time_us == standard_debounce_table[i].debounce_time_us)
		{	*debounce_value = standard_debounce_table[i].debounce_value;
			*debounce_select = 1;
			break;
		}
		else if(debounce_time_us == high_speed_debounce_table[i].debounce_time_us)			
		{	*debounce_value = high_speed_debounce_table[i].debounce_value;
			*debounce_select = 2;
			break;
		}
	}
	
/*  If debounce time was not found  */	
	if(i >= 16)
		error = MA203_ERROR_INVALID_DEBOUNCE_TIME;
		
Error:
	return error;
}

/*****************************************************************************
*	ma203_IDWriteByte
*
*	Write a command byte to the ID PROM of the instrument.
*	The instruments ID PROM is serially accessed
*****************************************************************************/
static Int32 ma203_IDWriteByte(MA203_HANDLE handle, UInt8 value)
{	Int32 error = MA203_SUCCESS;
	Int32 i;
	
/*  Initialize  */	
	checkErr(ma203_WriteRegister(handle, MA203_ID_ACCESS_REGISTER, 0));
	checkErr(ma203_WriteRegister(handle, MA203_ID_ACCESS_REGISTER, 0x4));
	
/*  Write start bit  */	
	checkErr(ma203_IDWriteBit(handle, 1));
	for(i=0; i<=7; i++)
	{
	/*  Write data bits  */		
		checkErr(ma203_IDWriteBit(handle, ((value &0x80)>>7)));
		value = value << 1;
	}
Error:	
	return error;
}

/*****************************************************************************
*	ma203_IDWriteBit
*
*	Write a single bit to the ID PROM of the instrument.
*	The instruments ID PROM is serially accessed
*****************************************************************************/
static Int32 ma203_IDWriteBit(MA203_HANDLE handle, Bool value)
{	Int32 error = MA203_SUCCESS;

/*  data with clock low  */
	checkErr(ma203_WriteRegister(handle, MA203_ID_ACCESS_REGISTER, (0x4 | (value & 0x1))));	

/*  data with clock high  */	 
	checkErr(ma203_WriteRegister(handle, MA203_ID_ACCESS_REGISTER, (0x6 | (value & 0x1))));	
Error:	
	return error;
}


/*****************************************************************************
*	ma203_IDReadByte
*
*	Reads a byte from the ID PROM of the instrument.
*	The instruments ID PROM is serially accessed
*****************************************************************************/
static Int32 ma203_IDReadByte(MA203_HANDLE handle, UInt8 *value)
{	Int32 error = MA203_SUCCESS;
	Int32 i;
	Bool data;
	
	*value = 0;
	for(i=7; i>=0; i--)
	{ 
	/*  read data 1 bit at a time  */	
		checkErr(ma203_IDReadBit(handle, &data));
		*value |= ((data & 0x1) << i);
	}
	
	
Error:	
	return error;
}


/*****************************************************************************
*	ma203_M_ModuleIDReadBit
*
*	Reads a single bit from the ID PROM of the instrument.
*	The instruments ID PROM is serially accessed
*****************************************************************************/
static Int32 ma203_IDReadBit(MA203_HANDLE handle, Bool *value)
{	Int32 error = MA203_SUCCESS;
	UInt16 data;

/*  clock low  */ 	
	checkErr(ma203_WriteRegister(handle, MA203_ID_ACCESS_REGISTER, 0x4));	
	
/*  clock high  */	
	checkErr(ma203_WriteRegister(handle, MA203_ID_ACCESS_REGISTER, 0x6));	
	
/*  Read data  */	
	checkErr(ma203_ReadRegister(handle, MA203_ID_ACCESS_REGISTER, &data));
	*value = (Bool)data;

Error:	
	return error;
}



/*****************************************************************************
*	Modify the functions below this point to use the MA203 driver in an 
*	environment not supported by an APIS platform support module. All functions
*	above this point should not have to be modified.
*
*	In general, any call to an APIS function (apis_func()) will need to be
*	modified to communicate with the instrument or operating system being used.
*
******************************************************************************/

/*****************************************************************************
*	ma203_Init
*
******************************************************************************/
Int32 ma203_Init (Int32 instr, Bool IDQuery, Bool reset, MA203_HANDLE* handle)
{	
	Int32 error = MA203_SUCCESS;
	Int32 warning = MA203_SUCCESS;
	UInt16 value;
  Int32 option_number;

//  Check parameters  
	checkParamRange(IDQuery, 0, 1, 2);
	checkParamRange(reset, 0, 1, 3);
	checkParamPointer(handle, 4);

// Allocate the data structure
	(*handle) = 0;
	(*handle) = malloc(sizeof(MA203_INSTR));
	if((*handle) == NULL)
	{	error = MA203_ERROR_OUT_OF_MEMORY;
		goto Error;
	}

	(*handle)->apis_handle = 0;
	apisCheckErrInit(apis_open((APIS_PATH)instr, &(*handle)->apis_handle));
  
// Perform ID Query 
	 if(IDQuery == MA203_TRUE)
	{	checkErr(ma203_GetIdentWord (*handle, 17, &value));
	  
		value &= 0x0FFF;
		if (value != (UInt16)MA203_VXI_MANF_ID) 
		{	error = MA203_ERROR_FAIL_ID_QUERY;
 			goto Error;
	  }

		checkErr(ma203_GetIdentWord (*handle, 18, &value));
		
		value &= 0x0FFF;
		if (value != (UInt16)MA203_VXI_MODEL_CODE) 
		{	error = MA203_ERROR_FAIL_ID_QUERY;
			goto Error;
		} 
	}

// Reset Instrument 
  if (reset == MA203_TRUE) 
  {	checkErr(ma203_Reset (*handle));
  }

	/*  Initialize the accessory module  */  
	checkErr(ma203_AccessoryModuleOption (*handle, &option_number, NULL));
	if((option_number == 4) || (option_number == 5))
		checkErr(ma203_AccessoryOption4Init(*handle));

Error:

	if((error == MA203_SUCCESS) && (warning != MA203_SUCCESS))	
		error = warning;
								
	return error; 
}

/*****************************************************************************
*	ma203_Close
*
*	Close the VISA session to the instrument.
******************************************************************************/
Int32 ma203_Close (MA203_HANDLE handle)
{	Int32 error = MA203_SUCCESS;
	if(handle->apis_handle != NULL)
		apisCheckErr(apis_close(handle->apis_handle));
	free(handle);
Error:
	return error;
}

/*****************************************************************************
*	ma203_reset
*	
*	Reset the instrument to it's default state
******************************************************************************/
Int32 ma203_Reset (MA203_HANDLE handle)
{	
	Int32 error = 0;
  Int32 option_number;

	apisCheckErr(apis_reset(handle->apis_handle));
	
	checkErr(ma203_AccessoryModuleOption (handle, &option_number, NULL));
	if((option_number == 4) || (option_number == 5))
		checkErr(ma203_AccessoryOption4Init (handle));
	else if(option_number == 6)
		checkErr(ma203_AccessoryOption6Init (handle));

Error:
	return error;
}



/*****************************************************************************
*	ma203_ReadRegister
*
*	Read the specified offset from the ma203's I/O space and returns
*	the result.
******************************************************************************/
Int32 ma203_ReadRegister (MA203_HANDLE handle, UInt16 offset, UInt16 *value)
{	Int32 error = MA203_SUCCESS;
	
/*  Check parameters  */
	checkParamPointer(value, 3);
	
	apisCheckErr(apis_read(handle->apis_handle, 2, offset, value)); // 2 is width (2 bytes)
	
Error:
	return error;
}



/*****************************************************************************
*	ma203_WriteRegister
*
*	Writes to the specified offset in the ma203's I/O space
******************************************************************************/
Int32 ma203_WriteRegister (MA203_HANDLE handle, UInt16 offset, UInt16 value)
{	Int32 error = MA203_SUCCESS;

	apisCheckErr(apis_write(handle->apis_handle, 2, offset, value));
Error:
	return error;
}	



/*****************************************************************************
*	ma203_InstallIrq
*	
*	Installs the interrupts handler to be called when an interrupt occurs
*	and configures the carrier to pass interrupts onto the system.
******************************************************************************/
Int32 ma203_InstallIrq (MA203_HANDLE handle, void *interruptHandler, Int32 vector, Int32 level, Int32 mode, void *var_ptr)
{	Int32 error = MA203_SUCCESS;

/*  Check parameters  */
	checkParamPointer(interruptHandler, 2);

	apisCheckErr(apis_irqinstall(handle->apis_handle, interruptHandler, vector, level, mode, var_ptr)); 

Error:
	return error;
}

/*****************************************************************************
*	ma203_UninstallIrq
*
*	Un-installs the interrupt handler for the instrument and disables
*	interrupt handling for the instrument
******************************************************************************/
Int32 ma203_UninstallIrq (MA203_HANDLE handle)
{	Int32 error = MA203_SUCCESS;

	apisCheckErr(apis_irqremove(handle->apis_handle));

Error:
	return error;
}


/*****************************************************************************
*	ma203_GetApisError
*
*	Returns the saved apis error code
******************************************************************************/
Int32 ma203_GetApisError (MA203_HANDLE handle, UInt32 *apis_error)
{	Int32 error = MA203_SUCCESS;
	*apis_error = handle->apis_error;
	return error;
}

